package org.xbib.interlibrary.client.hbz.z.legacy;

import org.xbib.datastructures.json.tiny.Json;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.common.util.InputStreamHelper;
import org.xbib.interlibrary.z.AbstractZSearchService;
import org.xbib.interlibrary.z.MarcHelper;
import org.xbib.marc.MarcField;
import org.xbib.marc.MarcRecord;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Supplier;
import java.util.logging.Level;
import java.util.logging.Logger;

import static org.xbib.interlibrary.z.MarcHelper.cleanTitle;

public class DisplayHelper {

    private static final Logger logger = Logger.getLogger(DisplayHelper.class.getName());

    private static final Map<String, Object> country = Objects.requireNonNull(getMap("/org/xbib/interlibrary/client/hbz/z/legacy/country.json"));

    private static final Map<String, Object> language = Objects.requireNonNull(getMap("/org/xbib/interlibrary/client/hbz/z/legacy/language.json"));

    private static final List<String> creatorFields = Arrays.asList(
            "100", "104", "108", "112", "116",
            "120", "124", "128", "132", "136",
            "140", "144", "148", "152", "156",
            "160", "164", "168", "172", "176",
            "180", "184", "188", "192", "196",
            "200", "204", "208", "212", "216",
            "220", "224", "228", "232", "236",
            "240", "244", "248", "252", "256",
            "260", "264", "268", "272", "276",
            "280", "284", "288", "292", "296");

    private static final List<String> titleFields = Arrays.asList(
            "310", "331", "335", "359", "360", "370"
    );

    private DisplayHelper() {
    }

    private static Map<String, Object> getMap(String path) {
        try {
            return Json.toMap(InputStreamHelper.readString(AbstractZSearchService.class.getResourceAsStream(path), StandardCharsets.UTF_8));
        } catch (IOException e) {
            logger.log(Level.WARNING, e.getMessage(), e);
            return null;
        }
    }

    public static void display(TinyMap.Builder<String, Object> builder,
                               MarcHelper marcHelper,
                               MarcRecord marcRecord,
                               String name,
                               String label,
                               String url,
                               Supplier<String> idSupplier) {
        putIf(builder, "servicename", name);
        putIf(builder, "servicelabel", label);
        putIf(builder, "serviceurl", url);
        putIf(builder, "id", idSupplier.get());
        putIf(builder, "date", marcHelper.extractFirstDate(marcRecord, "425", " "));
        putIf(builder, "creator", extractCreators(marcRecord));
        // contributor ?
        putIf(builder, "title", extractTitle(marcRecord));
        putIf(builder, "country", extractCountry(marcRecord,  "036", " "));
        putIf(builder, "language", extractLanguage(marcRecord, "037", " "));
        putIf(builder, "publication", marcHelper.extractPublication(marcRecord, List.of("403"), "a"));
        putIf(builder, "publishedAt", marcHelper.extractPublishedAt(marcRecord, List.of("419"), "a"));
        putIf(builder, "publishedBy", marcHelper.extractPublishedBy(marcRecord, List.of("419"), "b"));
        putIf(builder, "publishedOn", marcHelper.extractPublishedOn(marcRecord, List.of("419"), "c"));
        putIf(builder, "isbn", marcHelper.extractIBN(marcRecord, "540", List.of("a"), List.of(" ")));
        putIf(builder, "issn", marcHelper.extractIBN(marcRecord, "542", List.of("a"), List.of(" ")));
        // description
        // type
        putIf(builder, "format", marcHelper.extractFormatCodes(marcRecord));
        putIf(builder, "link", extractLinks(marcRecord));
        // in
        // chronology
        putIf(builder, "material", marcHelper.extractMaterialAsString(marcRecord));
        putIf(builder, "multipart", marcHelper.extractMultipart(marcRecord));
    }

    private static List<String> extractCreators(MarcRecord marcRecord) {
        List<String> list = new ArrayList<>();
        creatorFields.forEach(creatorFieldName ->
                marcRecord.getAll(creatorFieldName).forEach(f ->
                        f.getSubfields().stream().filter(sf -> " ".equals(sf.getId())).forEach(subfield ->
                                list.add(subfield.getValue()))));
        return list;
    }

    private static List<String> extractTitle(MarcRecord marcRecord) {
        List<String> list = new ArrayList<>();
        titleFields.forEach(titleField ->
                marcRecord.getAll(titleField).forEach(f ->
                        list.addAll(f.getSubfields().stream()
                                .map(sf -> cleanTitle(sf.getValue())).filter(MarcHelper::notEmpty).toList())));
        return list;
    }

    private static String extractCountry(MarcRecord marcRecord, String countryField, String subfieldId) {
        return marcRecord.getAll(countryField).stream().findFirst()
                .map(mf -> (String) country.get(mf.getFirstSubfieldValue(subfieldId))).orElse(null);
    }

    private static String extractLanguage(MarcRecord marcRecord, String languageField, String subfieldId) {
        return marcRecord.getAll(languageField).stream().findFirst()
                .map(mf -> (String) language.get(mf.getFirstSubfieldValue(subfieldId))).orElse(null);
    }

    private static List<Map<String, Object>> extractLinks(MarcRecord marcRecord) {
        List<Map<String, Object>> list = new ArrayList<>();
        for (MarcField marcField : marcRecord.getAll("655")) {
            TinyMap.Builder<String, Object> holdingsMap = TinyMap.builder();
            putIf(holdingsMap, "uri", marcField.getFirstSubfieldValue("u"));
            putIf(holdingsMap, "publicnote", marcField.getFirstSubfieldValue("x"));
            putIf(holdingsMap, "nonpublicnote", marcField.getFirstSubfieldValue("y"));
            putIf(holdingsMap, "access", marcField.getFirstSubfieldValue("z"));
            list.add(holdingsMap.build());
        }
        return list;
    }
    private static void putIf(Map<String, Object> map, String key, String value) {
        if (key != null && value != null && !"null".equals(value)) {
            map.put(key, value);
        }
    }

    private static void putIf(Map<String, Object> map, String key, Collection<?> value) {
        if (key != null && value != null && !value.isEmpty()) {
            map.put(key, value);
        }
    }

    private static void putIf(Map<String, Object> map, String key, boolean value) {
        if (key != null) {
            map.put(key, value);
        }
    }
}
